!MNH_LIC Copyright 1994-2018 CNRS, Meteo-France and Universite Paul Sabatier
!MNH_LIC This is part of the Meso-NH software governed by the CeCILL-C licence
!MNH_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt  
!MNH_LIC for details. version 1.
!
!     #######################
MODULE MODD_IBM_PARAM_n
  !   #######################
  !
  !****  MODD_IBM_PARAM_n - declaration of the control parameters
  !                         used in the immersed boundary method
  !
  !    PURPOSE
  !    -------
  !****  The purpose of this declarative module is to declare the constants
  !      which allow to initialize the embedded surface 
  !
  !
  !    IMPLICIT ARGUMENTS
  !    ------------------
  !      None 
  !
  !    REFERENCE
  !    ---------
  !          
  !    AUTHOR
  !    ------
  !	Franck Auguste (CERFACS-AE)
  !
  !    MODIFICATIONS
  !    -------------
  !      Original    01/01/2019
  !
  !----------------------------------------------------------------------------
  !
  !**** 0. DECLARATIONS
  !     ---------------
  !
  USE MODD_PARAMETERS, ONLY: JPMODELMAX
  IMPLICIT NONE
  !
  TYPE IBM_t
     !
     LOGICAL          :: LIBM,LIBM_TROUBLE                                                              ! IBM logical
     CHARACTER(LEN=6) :: CIBM_ADV                                                                       ! GCT switch
     CHARACTER(LEN=3) :: CIBM_MODE_INTE1_P,CIBM_MODE_INTE1_Q,CIBM_MODE_INTE1_R,CIBM_MODE_INTE1_S        ! 1D interpolation type
     CHARACTER(LEN=3) :: CIBM_MODE_INTE1_T,CIBM_MODE_INTE1_E
     CHARACTER(LEN=3) :: CIBM_MODE_INTE1NV,CIBM_MODE_INTE1TV,CIBM_MODE_INTE1CV
     CHARACTER(LEN=3) :: CIBM_MODE_INTE3_P,CIBM_MODE_INTE3_Q,CIBM_MODE_INTE3_R,CIBM_MODE_INTE3_S        ! 3D interpolation type
     CHARACTER(LEN=3) :: CIBM_MODE_INTE3_T,CIBM_MODE_INTE3_E,CIBM_MODE_INTE3_V
     CHARACTER(LEN=3) :: CIBM_TYPE_BOUND_P,CIBM_TYPE_BOUND_Q,CIBM_TYPE_BOUND_R,CIBM_TYPE_BOUND_S        ! Boundary interpolation type
     CHARACTER(LEN=3) :: CIBM_TYPE_BOUND_T,CIBM_TYPE_BOUND_E
     CHARACTER(LEN=3) :: CIBM_MODE_BOUND_P,CIBM_MODE_BOUND_Q,CIBM_MODE_BOUND_R,CIBM_MODE_BOUND_S        ! Boundary interpolation type
     CHARACTER(LEN=3) :: CIBM_MODE_BOUND_T,CIBM_MODE_BOUND_E
     CHARACTER(LEN=3) :: CIBM_FORC_BOUND_P,CIBM_FORC_BOUND_Q,CIBM_FORC_BOUND_R,CIBM_FORC_BOUND_S        ! Boundary interpolation type
     CHARACTER(LEN=3) :: CIBM_FORC_BOUND_T,CIBM_FORC_BOUND_E
     REAL             :: XIBM_FORC_BOUND_P,XIBM_FORC_BOUND_Q,XIBM_FORC_BOUND_R,XIBM_FORC_BOUND_S        ! Boundary interpolation type
     REAL             :: XIBM_FORC_BOUND_T,XIBM_FORC_BOUND_E
     CHARACTER(LEN=3) :: CIBM_TYPE_BOUNN_V,CIBM_TYPE_BOUNT_V,CIBM_TYPE_BOUNC_V                          ! BI type (velocity)
     CHARACTER(LEN=3) :: CIBM_MODE_BOUNN_V,CIBM_MODE_BOUNT_V,CIBM_MODE_BOUNC_V
     CHARACTER(LEN=3) :: CIBM_FORC_BOUNN_V,CIBM_FORC_BOUNT_V,CIBM_FORC_BOUNC_V,CIBM_FORC_BOUNR_V
     REAL             :: XIBM_FORC_BOUNN_V,XIBM_FORC_BOUNT_V,XIBM_FORC_BOUNC_V
     REAL             :: XIBM_RADIUS_P,XIBM_RADIUS_Q,XIBM_RADIUS_R,XIBM_RADIUS_S                        ! radius of IDW,MDW
     REAL             :: XIBM_RADIUS_T,XIBM_RADIUS_E,XIBM_RADIUS_V
     REAL             :: XIBM_POWERS_P,XIBM_POWERS_Q,XIBM_POWERS_R,XIBM_POWERS_S                        ! power of IDW,MDW
     REAL             :: XIBM_POWERS_T,XIBM_POWERS_E,XIBM_POWERS_V 
     INTEGER          :: NIBM_LAYER_V,NIBM_LAYER_T,NIBM_LAYER_P,NIBM_LAYER_R                            ! layers number in each type
     INTEGER          :: NIBM_LAYER_E,NIBM_LAYER_Q,NIBM_LAYER_S
     INTEGER          :: NIBM_ITR          ! maximum iteration in pressure solver                
     REAL :: XIBM_RUG ,XIBM_VISC, XIBM_CNU ! physical parameters for wall model
     REAL :: XIBM_EPSI                     ! min truncation parameters 
     REAL :: XIBM_IEPS                     ! max truncation parameters
     !
     REAL, DIMENSION(:,:,:)         , POINTER :: XIBM_XMUT=>NULL()    ! turbulent viscosity
     REAL, DIMENSION(:,:,:)         , POINTER :: XIBM_CURV=>NULL()    ! parameter for interface curvature
     REAL, DIMENSION(:,:,:,:)       , POINTER :: XIBM_LS=>NULL()      ! LSF for MNH
     REAL, DIMENSION(:,:,:,:)       , POINTER :: XIBM_SU=>NULL()      ! volume fraction based on LSF
     REAL, DIMENSION(:,:,:,:)       , POINTER :: XIBM_SUTR=>NULL()    ! volume fraction based on LSF if trouble
     REAL, DIMENSION(:,:,:,:)       , POINTER :: XIBM_GHOST_P=>NULL() ! Ghosts,Images location/Index
     REAL, DIMENSION(:,:,:,:)       , POINTER :: XIBM_GHOST_V=>NULL()
     REAL, DIMENSION(:,:,:,:,:)     , POINTER :: XIBM_IMAGE_P=>NULL()
     REAL, DIMENSION(:,:,:,:,:)     , POINTER :: XIBM_IMAGE_V=>NULL()
     INTEGER, DIMENSION(:,:,:,:)    , POINTER :: NIBM_GHOST_P=>NULL()
     INTEGER, DIMENSION(:,:,:,:)    , POINTER :: NIBM_GHOST_V=>NULL()
     INTEGER, DIMENSION(:,:,:,:,:,:), POINTER :: NIBM_IMAGE_P=>NULL()
     INTEGER, DIMENSION(:,:,:,:,:,:), POINTER :: NIBM_IMAGE_V=>NULL()
     REAL, DIMENSION(:,:,:,:,:,:)   , POINTER :: XIBM_TESTI_P=>NULL()
     REAL, DIMENSION(:,:,:,:,:,:)   , POINTER :: XIBM_TESTI_V=>NULL()
     !
  END TYPE IBM_t
  !
  TYPE(IBM_t), DIMENSION(JPMODELMAX), TARGET, SAVE :: IBM_MODEL
  !
  LOGICAL         , POINTER :: LIBM=>NULL()
  LOGICAL         , POINTER :: LIBM_TROUBLE=>NULL()
  CHARACTER(LEN=6), POINTER :: CIBM_ADV=>NULL()
  REAL            , POINTER :: XIBM_EPSI=>NULL()
  REAL            , POINTER :: XIBM_IEPS=>NULL()
  INTEGER         , POINTER :: NIBM_ITR=>NULL()
  !
  INTEGER, POINTER :: NIBM_LAYER_P=>NULL()
  INTEGER, POINTER :: NIBM_LAYER_Q=>NULL()
  INTEGER, POINTER :: NIBM_LAYER_R=>NULL()
  INTEGER, POINTER :: NIBM_LAYER_S=>NULL()
  INTEGER, POINTER :: NIBM_LAYER_T=>NULL()
  INTEGER, POINTER :: NIBM_LAYER_V=>NULL()
  INTEGER, POINTER :: NIBM_LAYER_E=>NULL()
  !
  REAL, POINTER :: XIBM_RADIUS_P=>NULL()
  REAL, POINTER :: XIBM_RADIUS_Q=>NULL()
  REAL, POINTER :: XIBM_RADIUS_R=>NULL()
  REAL, POINTER :: XIBM_RADIUS_S=>NULL()
  REAL, POINTER :: XIBM_RADIUS_T=>NULL()
  REAL, POINTER :: XIBM_RADIUS_E=>NULL()
  REAL, POINTER :: XIBM_RADIUS_V=>NULL()
  REAL, POINTER :: XIBM_POWERS_P=>NULL()
  REAL, POINTER :: XIBM_POWERS_Q=>NULL()
  REAL, POINTER :: XIBM_POWERS_R=>NULL()
  REAL, POINTER :: XIBM_POWERS_S=>NULL()
  REAL, POINTER :: XIBM_POWERS_T=>NULL()
  REAL, POINTER :: XIBM_POWERS_E=>NULL()
  REAL, POINTER :: XIBM_POWERS_V=>NULL()
  !
  REAL, POINTER :: XIBM_FORC_BOUNN_V=>NULL()
  REAL, POINTER :: XIBM_FORC_BOUNT_V=>NULL()
  REAL, POINTER :: XIBM_FORC_BOUNC_V=>NULL()
  REAL, POINTER :: XIBM_FORC_BOUND_P=>NULL()
  REAL, POINTER :: XIBM_FORC_BOUND_Q=>NULL()
  REAL, POINTER :: XIBM_FORC_BOUND_R=>NULL()
  REAL, POINTER :: XIBM_FORC_BOUND_S=>NULL()
  REAL, POINTER :: XIBM_FORC_BOUND_T=>NULL()
  REAL, POINTER :: XIBM_FORC_BOUND_E=>NULL()
  !
  CHARACTER(LEN=3), POINTER :: CIBM_MODE_INTE1_P=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_MODE_INTE3_P=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_MODE_BOUND_P=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_TYPE_BOUND_P=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_MODE_INTE1_Q=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_MODE_INTE3_Q=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_MODE_BOUND_Q=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_TYPE_BOUND_Q=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_MODE_INTE1_R=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_MODE_INTE3_R=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_MODE_BOUND_R=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_TYPE_BOUND_R=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_MODE_INTE1_S=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_MODE_INTE3_S=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_MODE_BOUND_S=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_TYPE_BOUND_S=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_MODE_INTE1_T=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_MODE_INTE3_T=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_MODE_BOUND_T=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_TYPE_BOUND_T=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_MODE_INTE1_E=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_MODE_INTE3_E=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_MODE_BOUND_E=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_TYPE_BOUND_E=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_MODE_INTE1NV=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_MODE_INTE1TV=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_MODE_INTE1CV=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_MODE_INTE3_V=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_MODE_BOUNN_V=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_TYPE_BOUNN_V=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_MODE_BOUNT_V=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_TYPE_BOUNT_V=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_MODE_BOUNC_V=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_TYPE_BOUNC_V=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_FORC_BOUNN_V=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_FORC_BOUNR_V=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_FORC_BOUNT_V=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_FORC_BOUNC_V=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_FORC_BOUND_P=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_FORC_BOUND_Q=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_FORC_BOUND_R=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_FORC_BOUND_S=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_FORC_BOUND_T=>NULL()
  CHARACTER(LEN=3), POINTER :: CIBM_FORC_BOUND_E=>NULL()
  !
  REAL, POINTER :: XIBM_RUG=>NULL()
  REAL, POINTER :: XIBM_VISC=>NULL()
  REAL, POINTER :: XIBM_CNU=>NULL()
  !
  REAL, DIMENSION(:,:,:)          , POINTER :: XIBM_XMUT=>NULL()
  REAL, DIMENSION(:,:,:)          , POINTER :: XIBM_CURV=>NULL()
  REAL, DIMENSION(:,:,:,:)        , POINTER :: XIBM_LS=>NULL()
  REAL, DIMENSION(:,:,:,:)        , POINTER :: XIBM_SU=>NULL()
  REAL, DIMENSION(:,:,:,:)        , POINTER :: XIBM_SUTR=>NULL()
  REAL, DIMENSION(:,:,:,:)        , POINTER :: XIBM_GHOST_P=>NULL()
  REAL, DIMENSION(:,:,:,:)        , POINTER :: XIBM_GHOST_V=>NULL()
  REAL, DIMENSION(:,:,:,:,:)      , POINTER :: XIBM_IMAGE_P=>NULL()
  REAL, DIMENSION(:,:,:,:,:)      , POINTER :: XIBM_IMAGE_V=>NULL()
  INTEGER, DIMENSION(:,:,:,:)     , POINTER :: NIBM_GHOST_P=>NULL()
  INTEGER, DIMENSION(:,:,:,:)     , POINTER :: NIBM_GHOST_V=>NULL()
  INTEGER, DIMENSION(:,:,:,:,:,:) , POINTER :: NIBM_IMAGE_P=>NULL()
  INTEGER, DIMENSION(:,:,:,:,:,:) , POINTER :: NIBM_IMAGE_V=>NULL()
  REAL, DIMENSION(:,:,:,:,:,:)    , POINTER :: XIBM_TESTI_P=>NULL()
  REAL, DIMENSION(:,:,:,:,:,:)    , POINTER :: XIBM_TESTI_V=>NULL()
  !
CONTAINS
  !
  SUBROUTINE IBM_GOTO_MODEL(KFROM, KTO)
    !
    INTEGER, INTENT(IN) :: KFROM, KTO
    !
    ! Save current state for allocated arrays
    IBM_MODEL(KFROM)%XIBM_XMUT=>XIBM_XMUT
    IBM_MODEL(KFROM)%XIBM_CURV=>XIBM_CURV
    IBM_MODEL(KFROM)%XIBM_LS=>XIBM_LS
    IBM_MODEL(KFROM)%XIBM_SU=>XIBM_SU
    IBM_MODEL(KFROM)%XIBM_GHOST_P=>XIBM_GHOST_P
    IBM_MODEL(KFROM)%XIBM_GHOST_V=>XIBM_GHOST_V
    IBM_MODEL(KFROM)%XIBM_GHOST_P=>XIBM_GHOST_P
    IBM_MODEL(KFROM)%XIBM_GHOST_V=>XIBM_GHOST_V
    IBM_MODEL(KFROM)%NIBM_IMAGE_P=>NIBM_IMAGE_P
    IBM_MODEL(KFROM)%NIBM_IMAGE_V=>NIBM_IMAGE_V
    IBM_MODEL(KFROM)%NIBM_IMAGE_P=>NIBM_IMAGE_P
    IBM_MODEL(KFROM)%NIBM_IMAGE_V=>NIBM_IMAGE_V
    IBM_MODEL(KFROM)%XIBM_TESTI_P=>XIBM_TESTI_P
    IBM_MODEL(KFROM)%XIBM_TESTI_V=>XIBM_TESTI_V
    !
    ! Current model is set to model KTO
    LIBM=>IBM_MODEL(KTO)%LIBM
    LIBM_TROUBLE=>IBM_MODEL(KTO)%LIBM_TROUBLE
    CIBM_ADV=>IBM_MODEL(KTO)%CIBM_ADV
    XIBM_EPSI=>IBM_MODEL(KTO)%XIBM_EPSI
    XIBM_IEPS=>IBM_MODEL(KTO)%XIBM_IEPS
    XIBM_RUG=>IBM_MODEL(KTO)%XIBM_RUG
    XIBM_VISC=>IBM_MODEL(KTO)%XIBM_VISC
    XIBM_CNU=>IBM_MODEL(KTO)%XIBM_CNU
    NIBM_ITR=>IBM_MODEL(KTO)%NIBM_ITR
    NIBM_LAYER_E=>IBM_MODEL(KTO)%NIBM_LAYER_E
    NIBM_LAYER_P=>IBM_MODEL(KTO)%NIBM_LAYER_P
    NIBM_LAYER_Q=>IBM_MODEL(KTO)%NIBM_LAYER_Q
    NIBM_LAYER_R=>IBM_MODEL(KTO)%NIBM_LAYER_R
    NIBM_LAYER_S=>IBM_MODEL(KTO)%NIBM_LAYER_S
    NIBM_LAYER_T=>IBM_MODEL(KTO)%NIBM_LAYER_T
    NIBM_LAYER_V=>IBM_MODEL(KTO)%NIBM_LAYER_V
    XIBM_XMUT=>IBM_MODEL(KTO)%XIBM_XMUT
    XIBM_CURV=>IBM_MODEL(KTO)%XIBM_CURV
    XIBM_LS=>IBM_MODEL(KTO)%XIBM_LS
    XIBM_SU=>IBM_MODEL(KTO)%XIBM_SU
    XIBM_SUTR=>IBM_MODEL(KTO)%XIBM_SUTR
    XIBM_GHOST_P=>IBM_MODEL(KTO)%XIBM_GHOST_P
    XIBM_GHOST_V=>IBM_MODEL(KTO)%XIBM_GHOST_V
    NIBM_GHOST_P=>IBM_MODEL(KTO)%NIBM_GHOST_P
    NIBM_GHOST_V=>IBM_MODEL(KTO)%NIBM_GHOST_V
    XIBM_IMAGE_P=>IBM_MODEL(KTO)%XIBM_IMAGE_P
    XIBM_IMAGE_V=>IBM_MODEL(KTO)%XIBM_IMAGE_V
    NIBM_IMAGE_P=>IBM_MODEL(KTO)%NIBM_IMAGE_P
    NIBM_IMAGE_V=>IBM_MODEL(KTO)%NIBM_IMAGE_V
    XIBM_TESTI_P=>IBM_MODEL(KTO)%XIBM_TESTI_P
    XIBM_TESTI_V=>IBM_MODEL(KTO)%XIBM_TESTI_V
    XIBM_RADIUS_P=>IBM_MODEL(KTO)%XIBM_RADIUS_P
    XIBM_POWERS_P=>IBM_MODEL(KTO)%XIBM_POWERS_P
    XIBM_RADIUS_Q=>IBM_MODEL(KTO)%XIBM_RADIUS_Q
    XIBM_POWERS_Q=>IBM_MODEL(KTO)%XIBM_POWERS_Q
    XIBM_RADIUS_R=>IBM_MODEL(KTO)%XIBM_RADIUS_R
    XIBM_POWERS_R=>IBM_MODEL(KTO)%XIBM_POWERS_R
    XIBM_RADIUS_S=>IBM_MODEL(KTO)%XIBM_RADIUS_S
    XIBM_POWERS_S=>IBM_MODEL(KTO)%XIBM_POWERS_S
    XIBM_RADIUS_T=>IBM_MODEL(KTO)%XIBM_RADIUS_T
    XIBM_POWERS_T=>IBM_MODEL(KTO)%XIBM_POWERS_T
    XIBM_RADIUS_E=>IBM_MODEL(KTO)%XIBM_RADIUS_E
    XIBM_POWERS_E=>IBM_MODEL(KTO)%XIBM_POWERS_E
    XIBM_RADIUS_V=>IBM_MODEL(KTO)%XIBM_RADIUS_V
    XIBM_POWERS_V=>IBM_MODEL(KTO)%XIBM_POWERS_V
    XIBM_FORC_BOUND_P => IBM_MODEL(KTO)%XIBM_FORC_BOUND_P
    XIBM_FORC_BOUND_Q => IBM_MODEL(KTO)%XIBM_FORC_BOUND_Q
    XIBM_FORC_BOUND_R => IBM_MODEL(KTO)%XIBM_FORC_BOUND_R
    XIBM_FORC_BOUND_S => IBM_MODEL(KTO)%XIBM_FORC_BOUND_S
    XIBM_FORC_BOUND_T => IBM_MODEL(KTO)%XIBM_FORC_BOUND_T
    XIBM_FORC_BOUND_E => IBM_MODEL(KTO)%XIBM_FORC_BOUND_E
    XIBM_FORC_BOUNN_V => IBM_MODEL(KTO)%XIBM_FORC_BOUNN_V
    XIBM_FORC_BOUNT_V => IBM_MODEL(KTO)%XIBM_FORC_BOUNT_V
    XIBM_FORC_BOUNC_V => IBM_MODEL(KTO)%XIBM_FORC_BOUNC_V
    CIBM_MODE_INTE1_P => IBM_MODEL(KTO)%CIBM_MODE_INTE1_P
    CIBM_MODE_INTE3_P => IBM_MODEL(KTO)%CIBM_MODE_INTE3_P
    CIBM_MODE_BOUND_P => IBM_MODEL(KTO)%CIBM_MODE_BOUND_P
    CIBM_TYPE_BOUND_P => IBM_MODEL(KTO)%CIBM_TYPE_BOUND_P
    CIBM_MODE_INTE1_Q => IBM_MODEL(KTO)%CIBM_MODE_INTE1_Q
    CIBM_MODE_INTE3_Q => IBM_MODEL(KTO)%CIBM_MODE_INTE3_Q
    CIBM_MODE_BOUND_Q => IBM_MODEL(KTO)%CIBM_MODE_BOUND_Q
    CIBM_TYPE_BOUND_Q => IBM_MODEL(KTO)%CIBM_TYPE_BOUND_Q
    CIBM_MODE_INTE1_R => IBM_MODEL(KTO)%CIBM_MODE_INTE1_R
    CIBM_MODE_INTE3_R => IBM_MODEL(KTO)%CIBM_MODE_INTE3_R
    CIBM_MODE_BOUND_R => IBM_MODEL(KTO)%CIBM_MODE_BOUND_R
    CIBM_TYPE_BOUND_R => IBM_MODEL(KTO)%CIBM_TYPE_BOUND_R
    CIBM_MODE_INTE1_S => IBM_MODEL(KTO)%CIBM_MODE_INTE1_S
    CIBM_MODE_INTE3_S => IBM_MODEL(KTO)%CIBM_MODE_INTE3_S
    CIBM_MODE_BOUND_S => IBM_MODEL(KTO)%CIBM_MODE_BOUND_S
    CIBM_TYPE_BOUND_S => IBM_MODEL(KTO)%CIBM_TYPE_BOUND_S
    CIBM_MODE_INTE1_T => IBM_MODEL(KTO)%CIBM_MODE_INTE1_T
    CIBM_MODE_INTE3_T => IBM_MODEL(KTO)%CIBM_MODE_INTE3_T
    CIBM_MODE_BOUND_T => IBM_MODEL(KTO)%CIBM_MODE_BOUND_T
    CIBM_TYPE_BOUND_T => IBM_MODEL(KTO)%CIBM_TYPE_BOUND_T
    CIBM_MODE_INTE1_E => IBM_MODEL(KTO)%CIBM_MODE_INTE1_E
    CIBM_MODE_INTE3_E => IBM_MODEL(KTO)%CIBM_MODE_INTE3_E
    CIBM_MODE_BOUND_E => IBM_MODEL(KTO)%CIBM_MODE_BOUND_E
    CIBM_TYPE_BOUND_E => IBM_MODEL(KTO)%CIBM_TYPE_BOUND_E
    CIBM_MODE_INTE1NV => IBM_MODEL(KTO)%CIBM_MODE_INTE1NV
    CIBM_MODE_INTE1TV => IBM_MODEL(KTO)%CIBM_MODE_INTE1TV
    CIBM_MODE_INTE1CV => IBM_MODEL(KTO)%CIBM_MODE_INTE1CV
    CIBM_MODE_INTE3_V => IBM_MODEL(KTO)%CIBM_MODE_INTE3_V
    CIBM_MODE_BOUNN_V => IBM_MODEL(KTO)%CIBM_MODE_BOUNN_V
    CIBM_TYPE_BOUNN_V => IBM_MODEL(KTO)%CIBM_TYPE_BOUNN_V
    CIBM_MODE_BOUNT_V => IBM_MODEL(KTO)%CIBM_MODE_BOUNT_V
    CIBM_TYPE_BOUNT_V => IBM_MODEL(KTO)%CIBM_TYPE_BOUNT_V
    CIBM_MODE_BOUNC_V => IBM_MODEL(KTO)%CIBM_MODE_BOUNC_V
    CIBM_TYPE_BOUNC_V => IBM_MODEL(KTO)%CIBM_TYPE_BOUNC_V
    CIBM_FORC_BOUNN_V => IBM_MODEL(KTO)%CIBM_FORC_BOUNN_V
    CIBM_FORC_BOUNR_V => IBM_MODEL(KTO)%CIBM_FORC_BOUNR_V
    CIBM_FORC_BOUNT_V => IBM_MODEL(KTO)%CIBM_FORC_BOUNT_V
    CIBM_FORC_BOUNC_V => IBM_MODEL(KTO)%CIBM_FORC_BOUNC_V
    CIBM_FORC_BOUND_P => IBM_MODEL(KTO)%CIBM_FORC_BOUND_P
    CIBM_FORC_BOUND_Q => IBM_MODEL(KTO)%CIBM_FORC_BOUND_Q
    CIBM_FORC_BOUND_R => IBM_MODEL(KTO)%CIBM_FORC_BOUND_R
    CIBM_FORC_BOUND_S => IBM_MODEL(KTO)%CIBM_FORC_BOUND_S
    CIBM_FORC_BOUND_T => IBM_MODEL(KTO)%CIBM_FORC_BOUND_T
    CIBM_FORC_BOUND_E => IBM_MODEL(KTO)%CIBM_FORC_BOUND_E
    !
  END SUBROUTINE IBM_GOTO_MODEL
  !
END MODULE MODD_IBM_PARAM_n
!
